use super::Solution;
use crate::utils::linked_list::ListNode;

pub struct MergeSortedLists {}

impl Solution for MergeSortedLists {
    fn solution(
        &self,
        input: &mut Vec<String>,
    ) -> Result<Box<dyn std::fmt::Display + Sync>, Box<dyn std::error::Error>> {
        let lists = parse_input(input.first().unwrap());
        let merged_list = self.merge_two_lists(lists.0, lists.1);
        Ok(Box::new(format!("{:?}", ListNode::to_vec(merged_list)).replace(" ", "")))
    }

    fn get_id(&self) -> u32 {
        21
    }
}

impl MergeSortedLists {
    pub fn merge_two_lists(
        &self,
        list1: Option<Box<ListNode>>,
        list2: Option<Box<ListNode>>,
    ) -> Option<Box<ListNode>> {
        match (list1, list2) {
            (Some(l1), None) => Some(l1),
            (None, Some(l2)) => Some(l2),
            (None, None)     => None,
            (Some(l1), Some(l2)) => match l1.val <= l2.val {
                true  => Some(Box::new(ListNode {
                    val: l1.val,
                    next: self.merge_two_lists(l1.next, Some(l2)),
                })),
                false => Some(Box::new(ListNode {
                    val: l2.val,
                    next: self.merge_two_lists(Some(l1), l2.next),
                })),
            },
        }
    }
}

fn parse_input(input: &str) -> (Option<Box<ListNode>>, Option<Box<ListNode>>) {
    let parts: Vec<&str> = input.split("],").collect();
    let list1_str = parts[0].split('=').nth(1).unwrap().trim().to_string() + "]";
    let list2_str = parts[1].split('=').nth(1).unwrap().trim().to_string() + "]";

    let list1 = ListNode::from_vec(ListNode::parse_list(&list1_str));
    let list2 = ListNode::from_vec(ListNode::parse_list(&list2_str));

    (list1, list2)
}


/// Test from puzzle input
#[cfg(test)]
mod test {
    use super::*;
    use crate::*;

    #[test]
    fn test1() {
        let solution = merge_sorted_lists::MergeSortedLists {};

        //Complete the Challenge
        let answer = solution
            .solution(
                utils::get_input(solution.get_id(), utils::InputType::Test1)
                    .unwrap()
                    .as_mut(),
            )
            .unwrap()
            .to_string();

        assert_eq!(answer, "[1,1,2,3,4,4]");
    }

    #[test]
    fn test2() {
        let solution = merge_sorted_lists::MergeSortedLists {};

        //Complete the Challenge
        let answer = solution
            .solution(
                utils::get_input(solution.get_id(), utils::InputType::Test2)
                    .unwrap()
                    .as_mut(),
            )
            .unwrap()
            .to_string();

        assert_eq!(answer, "[]");
    }

    #[test]
    fn test3() {
        let solution = merge_sorted_lists::MergeSortedLists {};

        //Complete the Challenge
        let answer = solution
            .solution(
                utils::get_input(solution.get_id(), utils::InputType::Test3)
                    .unwrap()
                    .as_mut(),
            )
            .unwrap()
            .to_string();

        assert_eq!(answer, "[0]");
    }
}
